@@ -7,8 +7,11 @@ const RDF = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';
 const XSD = 'http://www.w3.org/2001/XMLSchema#';
 
 module.exports = {
+  // TODO: Deprecated and will be removed later. Use LINK_HEADER_CONTEXT.
   LINK_HEADER_REL: 'http://www.w3.org/ns/json-ld#context',
 
+  LINK_HEADER_CONTEXT: 'http://www.w3.org/ns/json-ld#context',
+
   RDF,
   RDF_LIST: RDF + 'List',
   RDF_FIRST: RDF + 'first',
@@ -17,6 +20,7 @@ module.exports = {
   RDF_TYPE: RDF + 'type',
   RDF_PLAIN_LITERAL: RDF + 'PlainLiteral',
   RDF_XML_LITERAL: RDF + 'XMLLiteral',
+  RDF_JSON_LITERAL: RDF + 'JSON',
   RDF_OBJECT: RDF + 'object',
   RDF_LANGSTRING: RDF + 'langString',
 
