@@ -47,8 +47,8 @@ api.createMergedNodeMap = (input, options) => {
 api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
   // recurse through array
   if(types.isArray(input)) {
-    for(let i = 0; i < input.length; ++i) {
-      api.createNodeMap(input[i], graphs, graph, issuer, undefined, list);
+    for(const node of input) {
+      api.createNodeMap(node, graphs, graph, issuer, undefined, list);
     }
     return;
   }
@@ -74,6 +74,11 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
       list.push(input);
     }
     return;
+  } else if(list && graphTypes.isList(input)) {
+    const _list = [];
+    api.createNodeMap(input['@list'], graphs, graph, issuer, name, _list);
+    list.push({'@list': _list});
+    return;
   }
 
   // Note: At this point, input must be a subject.
@@ -81,8 +86,7 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
   // spec requires @type to be named first, so assign names early
   if('@type' in input) {
     const types = input['@type'];
-    for(let i = 0; i < types.length; ++i) {
-      const type = types[i];
+    for(const type of types) {
       if(type.indexOf('_:') === 0) {
         issuer.getId(type);
       }
@@ -105,9 +109,7 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
   const subject = subjects[name] = subjects[name] || {};
   subject['@id'] = name;
   const properties = Object.keys(input).sort();
-  for(let pi = 0; pi < properties.length; ++pi) {
-    let property = properties[pi];
-
+  for(let property of properties) {
     // skip @id
     if(property === '@id') {
       continue;
@@ -119,8 +121,7 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
       const reverseMap = input['@reverse'];
       for(const reverseProperty in reverseMap) {
         const items = reverseMap[reverseProperty];
-        for(let ii = 0; ii < items.length; ++ii) {
-          const item = items[ii];
+        for(const item of items) {
           let itemName = item['@id'];
           if(graphTypes.isBlankNode(item)) {
             itemName = issuer.getId(itemName);
@@ -144,6 +145,12 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
       continue;
     }
 
+    // recurse into included
+    if(property === '@included') {
+      api.createNodeMap(input[property], graphs, graph, issuer);
+      continue;
+    }
+
     // copy non-@type keywords
     if(property !== '@type' && isKeyword(property)) {
       if(property === '@index' && property in subject &&
@@ -171,9 +178,7 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
       util.addValue(subject, property, [], {propertyIsArray: true});
       continue;
     }
-    for(let oi = 0; oi < objects.length; ++oi) {
-      let o = objects[oi];
-
+    for(let o of objects) {
       if(property === '@type') {
         // rename @type blank nodes
         o = (o.indexOf('_:') === 0) ? issuer.getId(o) : o;
@@ -181,6 +186,11 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
 
       // handle embedded subject or subject reference
       if(graphTypes.isSubject(o) || graphTypes.isSubjectReference(o)) {
+        // skip null @id
+        if('@id' in o && !o['@id']) {
+          continue;
+        }
+
         // relabel blank node @id
         const id = graphTypes.isBlankNode(o) ?
           issuer.getId(o['@id']) : o['@id'];
@@ -190,6 +200,10 @@ api.createNodeMap = (input, graphs, graph, issuer, name, list) => {
           subject, property, {'@id': id},
           {propertyIsArray: true, allowDuplicate: false});
         api.createNodeMap(o, graphs, graph, issuer, id);
+      } else if(graphTypes.isValue(o)) {
+        util.addValue(
+          subject, property, o,
+          {propertyIsArray: true, allowDuplicate: false});
       } else if(graphTypes.isList(o)) {
         // handle @list
         const _list = [];
@@ -227,7 +241,7 @@ api.mergeNodeMapGraphs = graphs => {
       const mergedNode = merged[id];
 
       for(const property of Object.keys(node).sort()) {
-        if(isKeyword(property)) {
+        if(isKeyword(property) && property !== '@type') {
           // copy keywords
           mergedNode[property] = util.clone(node[property]);
         } else {
@@ -249,8 +263,7 @@ api.mergeNodeMaps = graphs => {
   // add all non-default graphs to default graph
   const defaultGraph = graphs['@default'];
   const graphNames = Object.keys(graphs).sort();
-  for(let i = 0; i < graphNames.length; ++i) {
-    const graphName = graphNames[i];
+  for(const graphName of graphNames) {
     if(graphName === '@default') {
       continue;
     }
@@ -265,9 +278,8 @@ api.mergeNodeMaps = graphs => {
       subject['@graph'] = [];
     }
     const graph = subject['@graph'];
-    const ids = Object.keys(nodeMap).sort();
-    for(let ii = 0; ii < ids.length; ++ii) {
-      const node = nodeMap[ids[ii]];
+    for(const id of Object.keys(nodeMap).sort()) {
+      const node = nodeMap[id];
       // only add full subjects
       if(!graphTypes.isSubjectReference(node)) {
         graph.push(node);
