@@ -6,7 +6,9 @@
  * Copyright 2010-2017 Digital Bazaar, Inc.
  */
 const path = require('path');
-const webpackMerge = require('webpack-merge');
+const {
+  merge: webpackMerge
+} = require('webpack-merge');
 
 // build multiple outputs
 module.exports = [];
@@ -14,39 +16,44 @@ module.exports = [];
 // custom setup for each output
 // all built files will export the "jsonld" library but with different content
 const outputs = [
-  // core jsonld library
+  // core jsonld library (standard)
+  // larger version for wide compatibilty
   {
     entry: [
-      // 'babel-polyfill' is very large, list features explicitly
-      'core-js/features/array/from',
-      'core-js/features/array/includes',
-      'core-js/features/map',
-      'core-js/features/object/assign',
-      'core-js/features/object/entries',
-      'core-js/features/promise',
-      'core-js/features/set',
-      'core-js/features/string/starts-with',
-      'core-js/features/symbol',
       // main lib
       './lib/index.js'
     ],
-    filenameBase: 'jsonld'
+    filenameBase: 'jsonld',
+    targets: {
+      // use slightly looser browserslist defaults
+      browsers: 'defaults, > 0.25%'
+    }
   },
-  /*
-  // core jsonld library + extra utils and networking support
+  // core jsonld library (esm)
+  // smaller version using features from browsers with ES Modules support
   {
-    entry: ['./lib/index.all.js'],
-    filenameBase: 'jsonld.all'
-  }
-  */
-  // custom builds can be created by specifying the high level files you need
-  // webpack will pull in dependencies as needed
-  // Note: if using UMD or similar, add jsonld.js *last* to properly export
-  // the top level jsonld namespace.
+    entry: [
+      // main lib
+      './lib/index.js'
+    ],
+    filenameBase: 'jsonld.esm',
+    targets: {
+      esmodules: true
+    }
+  },
+  // - custom builds can be created by specifying the high level files you need
+  // - webpack will pull in dependencies as needed
+  // - Note: if using UMD or similar, add jsonld.js *last* to properly export
+  //   the top level jsonld namespace.
+  // - see Babel and browserslist docs for targets
   //{
   //  entry: ['./lib/FOO.js', ..., './lib/jsonld.js'],
   //  filenameBase: 'jsonld.custom'
-  //  libraryTarget: 'umd'
+  //  libraryTarget: 'umd',
+  //  targets: {
+  //    // for example, just target latest browsers for development
+  //    browsers: 'last 1 chrome version, last 1 firefox version',
+  //  }
   //}
 ];
 
@@ -57,6 +64,10 @@ outputs.forEach(info => {
     entry: {
       jsonld: info.entry
     },
+    // enable for easier debugging
+    //optimization: {
+    //  minimize: false
+    //},
     module: {
       rules: [
         {
@@ -69,13 +80,26 @@ outputs.forEach(info => {
             include: [
               /(node_modules\/canonicalize)/,
               /(node_modules\/lru-cache)/,
-              /(node_modules\/rdf-canonize)/
+              /(node_modules\/rdf-canonize)/,
+              /(node_modules\/yallist)/
             ]
           }],
           use: {
             loader: 'babel-loader',
             options: {
-              presets: ['@babel/preset-env'],
+              presets: [
+                [
+                  '@babel/preset-env',
+                  {
+                    useBuiltIns: 'usage',
+                    corejs: '3.9',
+                    // TODO: remove for babel 8
+                    bugfixes: true,
+                    //debug: true,
+                    targets: info.targets
+                  }
+                ]
+              ],
               plugins: [
                 [
                   '@babel/plugin-proposal-object-rest-spread',
@@ -128,21 +152,7 @@ outputs.forEach(info => {
       library: info.library || '[name]',
       libraryTarget: info.libraryTarget || 'umd'
     },
-    devtool: 'cheap-module-source-map',
-    plugins: [
-      /*
-      new webpack.optimize.UglifyJsPlugin({
-        //beautify: true,
-        compress: {
-          warnings: true
-        },
-        output: {
-          comments: false
-        },
-        sourceMap: true
-      })
-      */
-    ]
+    devtool: 'cheap-module-source-map'
   });
   if(info.library === null) {
     delete minify.output.library;
